package cn.wzpmc.hny.games;

import cn.wzpmc.hny.HappyNewLobby;
import cn.wzpmc.hny.pojo.CustomItem;
import cn.wzpmc.hny.pojo.GlobalPlayerData;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.format.TextDecoration;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.Server;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.InventoryType;
import org.bukkit.inventory.Inventory;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class PlayerDataManager {
    private final static NamespacedKey moneyKey = new NamespacedKey("hny", "money");
    private final static NamespacedKey itemKey = new NamespacedKey("hny", "items");
    private static PlayerDataManager instance;
    private final Map<Player, GlobalPlayerData> players;
    private final Server server;
    private PlayerDataManager(){
        this.players = new HashMap<>();
        this.server = HappyNewLobby.getPlugin(HappyNewLobby.class).getServer();
    }
    public void initPlayerData(Player player){
        PersistentDataContainer persistentDataContainer = player.getPersistentDataContainer();
        Double money = persistentDataContainer.get(moneyKey, PersistentDataType.DOUBLE);
        if (money == null){
            money = 0d;
        }
        int[] itemIds = persistentDataContainer.get(itemKey, PersistentDataType.INTEGER_ARRAY);
        if (itemIds == null){
            itemIds = new int[]{};
        }
        List<CustomItem> items = new ArrayList<>();
        for (int itemId : itemIds) {
            items.add(CustomItem.ItemStatic.getItem(itemId));
        }
        GlobalPlayerData globalPlayerData = new GlobalPlayerData(money, items);
        this.players.put(player, globalPlayerData);
    }
    public GlobalPlayerData getPlayerData(Player player) {
        return this.players.get(player);
    }
    public static final Component PLAYER_STATISTICAL_INVENTORY_TITLE = Component.text("统计数据").color(NamedTextColor.RED);
    public static final Component PLAYER_INVENTORY_TITLE = Component.text("你的道具仓库").color(NamedTextColor.DARK_RED);
    public static final Component ACCOUNT_BUTTON_DISPLAY_NAME = Component.text("你的账户").color(NamedTextColor.GREEN);
    public void showPlayerData(Player player){
        Inventory inventory = this.server.createInventory(player, InventoryType.CHEST, PLAYER_STATISTICAL_INVENTORY_TITLE);
        GlobalPlayerData globalPlayerData = this.players.get(player);
        double money = globalPlayerData.getMoney();
        CustomItem moneyItem = new CustomItem(Material.EMERALD, ACCOUNT_BUTTON_DISPLAY_NAME).lore(Component.text("当前金额：").append(Component.text(money)));
        inventory.setItem(11, moneyItem.getStack(1));
        inventory.setItem(15, CustomItem.ProjectItemStatic.INTO_INVENTORY_BUTTON.getStack(1));
        player.openInventory(inventory);
    }
    public void showPlayerInventory(Player player){
        Inventory inventory = this.server.createInventory(player, InventoryType.CHEST, PLAYER_INVENTORY_TITLE);
        GlobalPlayerData globalPlayerData = this.players.get(player);
        for (CustomItem customItem : globalPlayerData.getInventory()) {
            inventory.addItem(customItem.getStack(1));
        }
        inventory.setItem(26, CustomItem.ProjectItemStatic.INVENTORY_BACK_BUTTON.getStack(1));
        player.openInventory(inventory);
    }
    public static PlayerDataManager getInstance(){
        if (instance == null){
            instance = new PlayerDataManager();
        }
        return instance;
    }

    public void resetAllData() {
        for (Player onlinePlayer : server.getOnlinePlayers()) {
            PersistentDataContainer persistentDataContainer = onlinePlayer.getPersistentDataContainer();
            persistentDataContainer.set(moneyKey, PersistentDataType.DOUBLE, 0d);
            persistentDataContainer.set(itemKey, PersistentDataType.INTEGER_ARRAY, new int[0]);
        }
    }
    public void savePlayerData(Player player) {
        GlobalPlayerData globalPlayerData = this.players.get(player);
        List<CustomItem> inventory = globalPlayerData.getInventory();
        double money = globalPlayerData.getMoney();
        int[] ids = new int[inventory.size()];
        int i = 0;
        for (CustomItem customItem : inventory) {
            Integer id = CustomItem.ItemStatic.getId(customItem);
            ids[i++] = id;
        }
        PersistentDataContainer persistentDataContainer = player.getPersistentDataContainer();
        persistentDataContainer.set(moneyKey, PersistentDataType.DOUBLE, money);
        persistentDataContainer.set(itemKey, PersistentDataType.INTEGER_ARRAY, ids);
    }
}
