package cn.wzpmc.hny.pojo;

import lombok.AllArgsConstructor;
import lombok.Getter;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.format.TextDecoration;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import java.util.*;

@AllArgsConstructor
public final class CustomItem {
    public final Material originalMaterial;
    @Getter
    public final Component displayName;
    @Getter
    public final List<Component> lore;
    public final int customModelId;
    public CustomItem(Material originalMaterial, Component displayName, List<Component> lore){
        this(originalMaterial, displayName, lore, -1);
    }
    public CustomItem(Material originalMaterial, Component displayName){
        this(originalMaterial, displayName, -1);
    }
    public CustomItem(Material originalMaterial, Component displayName, int customModelId){
        this(originalMaterial, displayName, new ArrayList<>(), customModelId);
    }
    public ItemStack getStack(int count){
        ItemStack itemStack = new ItemStack(this.originalMaterial, count);
        ItemMeta itemMeta = itemStack.getItemMeta();
        if (this.customModelId != -1){
            itemMeta.setCustomModelData(this.customModelId);
        }
        itemMeta.displayName(this.displayName);
        itemMeta.lore(this.lore);
        itemStack.setItemMeta(itemMeta);
        return itemStack;
    }
    public CustomItem lore(Component component){
        this.lore.add(component.decoration(TextDecoration.ITALIC, false).decorate(TextDecoration.BOLD));
        return this;
    }
    public CustomItem description(Component component){
        this.lore.add(component.decorate(TextDecoration.ITALIC));
        return this;
    }
    public boolean is(ItemStack itemStack){
        ItemMeta itemMeta = itemStack.getItemMeta();
        return itemStack.getType().equals(this.originalMaterial) && Objects.equals(itemMeta.lore(), this.lore) && Objects.equals(itemMeta.displayName(), this.displayName);
    }
    public final static class ItemStatic {
        public static final Map<Integer, CustomItem> items = new HashMap<>();
        public static final Map<CustomItem, Integer> ids = new HashMap<>();
        public static final CustomItem DISH = new CustomItem(Material.MUSIC_DISC_5, Component.text("天外重声大碟"))
                .lore(Component.text("复活后，6s内不会受到伤害及负面效果影响。"))
                .lore(Component.text("由于其他道具的特殊效果复活时，该效果的持续事件变为原时常的一半。"))
                .description(Component.text("你觉得这曲子怎么样？"))
                .description(Component.text("还行，如果没有后面那段"))
                .description(Component.text("哪段？"))
                .description(Component.text("听下面五段对话......"));//30
        public static final CustomItem HEART = new CustomItem(Material.ENCHANTED_GOLDEN_APPLE, Component.text("彩虹生命"))
                .lore(Component.text("受到伤害时，无效那次伤害并回复那个伤害数值一半的生命值。"));//20
        public static final CustomItem STRENGTH = new CustomItem(Material.FLINT_AND_STEEL, Component.text("力量焊接"))
                .lore(Component.text("下次造成伤害时，对对方造成的伤害翻倍，随后自身受到此效果增加的伤害数值的伤害"))
                .description(Component.text("夕阳 电子终结龙 16000.jpg"));//40
        public static final CustomItem BADLY_HURT = new CustomItem(Material.NETHERITE_AXE, Component.text("重伤"))
                .lore(Component.text("下次击中玩家时，为其施加 5s 的「重伤」效果"))
                .lore(Component.text("重伤："))
                .lore(Component.text("在该效果持续时间内，降低玩家攻击伤害(虚弱1)，且不能在此期间使用治疗类道具"));//20
        public static final CustomItem TOTEM_OF_UNDYING = new CustomItem(Material.TOTEM_OF_UNDYING, Component.text("不死图腾"))
                .lore(Component.text("下一次受到致命伤害时，不会死亡，并将血量恢复至 4 点"))
                .lore(Component.text("由于该效果触发的「天外重声大碟」效果持续时间缩短为原时长的一半。"))
                .description(Component.text("「那他死了吗？」"))
                .description(Component.text("「如死」"));//60
        public static final CustomItem RESPAWN_BEACON = new CustomItem(Material.BEACON, Component.text("重生信标"))
                .lore(Component.text("仅能在主要阶段 Ⅲ 购买、使用"))
                .lore(Component.text("复活一名玩家，每个复活信标限用一次。"))
                .lore(Component.text("由于该效果触发的「天外重声大碟」的效果持续时间缩短为原时长的一半。"))
                .description(Component.text("你捡我卡啊，你自己去有**用"))
                .description(Component.text("这是新年活动，不用捡卡，别惦记你那Apex了"))
                .description(Component.text("可是我真的在玩Apex啊"))
                .description(Component.text("你**摸鱼是吧？"));//80
        public static final CustomItem TREAT = new CustomItem(Material.GOLDEN_APPLE, Component.text("治疗包"))
                .lore(Component.text("使用后一段时间，回复相当于当前最大生命值的 25%(5s)/30%(7s)/50% (10s)，同时清除所有1个/2个/所有负面效果，在一段时间内不能使用治疗类道具(缓速3)"))
                .description(Component.text("正在使用凤凰治疗包"));//10
        public static final CustomItem MARK = new CustomItem(Material.NETHER_STAR, Component.text("标记"))
                .lore(Component.text("为一名玩家施加30s的发光效果"));//10
        public static final CustomItem TURBINE = new CustomItem(Material.TURTLE_HELMET, Component.text("涡轮"))
                .lore(Component.text("赋予自己速度2与跳跃提升1,持续5s"))
                .description(Component.text("你说得对，但涡轮是一种wzp..."));//15
        public static final CustomItem MILK = new CustomItem(Material.MILK_BUCKET, Component.text("牛奶"))
                .lore(Component.text("清除负面效果"))
                .description(Component.text("这不就原版吗"));//10
        public static final CustomItem PUNCTURE = new CustomItem(Material.TRIDENT, Component.text("戳穿"))
                .lore(Component.text("本局游戏仅能购买 n 次（n 未确定，随人数变化而变化，全体玩家共用次数）"))
                .lore(Component.text("选择一名玩家，查看其阵营（有 30% 概率失败）"))
                .description(Component.text("「那你是怎么知道的？」"))
                .description(Component.text("「用望远镜看鸽子显示器」"));//25
        public static final CustomItem WHITEWASH = new CustomItem(Material.NETHERITE_PICKAXE, Component.text("粉饰"))
                .lore(Component.text("当其他玩家对你使用戳穿时，直接判定获取失败"))
                .description(Component.text("「那你又是怎么不让他们知道的？」"))
                .description(Component.text("「把鸽子显示器砸了」"));//30
        public static final CustomItem SURVEILLANCE = new CustomItem(Material.ENDER_EYE, Component.text("监视"))
                .lore(Component.text("以一名玩家为对象使用，该玩家购买物品时你会获得通知"))
                .description(Component.text("所以你是怎么拿到的？"))
                .description(Component.text("「我已阅读并同意《用户许可协议》《隐私政策》《第三方数据共享协议》」"))
                .description(Component.text("我现在退出还来得及吗..."));//30
        public static final CustomItem EVOLUTION_CRYSTAL = new CustomItem(Material.END_CRYSTAL, Component.text("进化水晶"))
                .lore(Component.text("仅限防守方，功能类、实体类(ATK 0 ，HP 10 ， DEF 10()"))
                .lore(Component.text("使用后，召唤一个进化水晶"))
                .lore(Component.text("实体效果："))
                .lore(Component.text("  为 30 格内的末影龙和放置玩家恢复生命值（1/s），死亡时发生爆炸"))
                .description(Component.text("眼前一黑，进化创飞！"))
                .description(Component.text("这不是炸飞的吗..."))
                .description(Component.text("就你话多！"));//40
        public static final Component EVOLUTION_CRYSTAL_NAME = Component.text("进化水晶").color(NamedTextColor.DARK_RED);
        public static final NamespacedKey EVOLUTION_CRYSTAL_PLAYER_KEY = new NamespacedKey("hny", "placer");
        public static final CustomItem FU_CARD = new CustomItem(Material.RED_DYE, Component.text("福卡"))
                .lore(Component.text("在最终阶段开始时，获得 150 制造材料"))
                .description(Component.text("你五福集了吗？"))
                .description(Component.text("还没，马上"))
                .description(Component.text("那你开原神干啥？"))
                .description(Component.text("集五「芙」啊"))
                .description(Component.text("大过年就不要玩谐音梗了啊！"));//寻找
        public static final CustomItem TIME_WATCH = new CustomItem(Material.CLOCK, Component.text("时间洪流怀表"))
                .lore(Component.text("使用后，从以下效果中选择一个："))
                .lore(Component.text("  ->倒计时增加 5 分钟"))
                .lore(Component.text("  ->倒计时减少 5 分钟"))
                .lore(Component.text("只能在每个阶段开始后 10 分钟内使用"))
                .description(Component.text("俗话说「一寸光阴一寸金，寸金难买寸光阴」，这告诉我们什么道理？"))
                .description(Component.text("要用制造材料！"));//60
        public static final CustomItem PILL = new CustomItem(Material.SPLASH_POTION, Component.text("药丸"))
                .lore(Component.text("使用后，减少 15 侵蚀度"))
                .description(Component.text("这东西最好别多吃"))
                .description(Component.text("？副作用很大吗"))
                .description(Component.text("不，因为你「吃枣药丸」"))
                .description(Component.text("你不玩谐音梗是会死吗？"));//15
        public static final CustomItem PUT_OFF = new CustomItem(Material.COMPASS, Component.text("计划推迟"))
                .lore(Component.text("在受到致命伤害时，不会立即死亡，且期间不会受到伤害。20s后死亡。"))
                .description(Component.text("2月1日，给流萤攒星琼，摆一天"))
                .description(Component.text("2月2日，原神新春会，摆一天"))
                .description(Component.text("2月3日，鸽子啊鸽子，你怎能如此堕落，新年活动你不做了吗"))
                .description(Component.text("2月4日，什么时候开始跃迁啊，我现在就要去匹诺康尼啊啊啊啊啊啊"));//50
        public static final CustomItem CODE = new CustomItem(Material.TNT_MINECART, Component.text("抛出异常的代码"))
                .lore(Component.text("玩家对其他实体造成伤害时，有概率为那个实体附加「异常」效果。"))
                .lore(Component.text("「异常」"))
                .lore(Component.text("  持有此效果的实体攻击其他实体时，有 75% 的概率使本次攻击无效化。此效果不能叠加。"))
                .description(Component.text("他只需一行代码，就能让服务器瘫痪；再给他一个写插件的机会，他能让关服成为压轴节目。他就是与「Helper」紫线齐名的「Sleeper」七彩精华，就连五星评论家麦克阿瑟也表示「即使是上帝亲自使用他的插件，服务端也跑不起来一点」。他究竟是谁，竟能写出如此逆天的代码，大型纪录片《七彩精华传奇》持续为您播出。"));//35
        public static final CustomItem SEED = new CustomItem(Material.WHEAT_SEEDS, Component.text("制造材料种子"))
                .lore(Component.text("每分钟获得 10 制造材料，持续10分钟，死亡消失"))
                .description(Component.text("寄！精华没了，做不了种子了！"))
                .description(Component.text("一刻也没有为精华的用尽哀悼，立刻来到现场的是七彩精华！"));//30
        public static final CustomItem SHIELD = new CustomItem(Material.SHIELD, Component.text("盾牌"))
                .lore(Component.text("普通的盾牌，没什么稀奇的"))
                .description(Component.text("所以为什么这么多游戏都有矛和盾啊"))
                .description(Component.text("因为矛盾具有普遍性"))
                .description(Component.text("那为啥都不太一样啊"))
                .description(Component.text("因为矛盾具有特殊性"));//30
        public static final CustomItem HELMETS_AND_SHOES = new CustomItem(Material.LEATHER_HELMET, Component.text("头+鞋"))
                .lore(Component.text("购买 15 制造材料，升级需要 20 制造材料"))
                .description(Component.text("随着你穿上这件衣服，一轮强劲的音乐响起"))
                .description(Component.text("「浙江温州，浙江温州，江南皮革厂倒闭了」"));
        public static final CustomItem ARMOR_AND_PANTS = new CustomItem(Material.LEATHER_CHESTPLATE, Component.text("盔甲+裤子"))
                .lore(Component.text("购买 20 制造材料，升级需要 30 制造材料"))
                .description(Component.text("抚摸着衣服，一股不属于这个世界的声音传来"))
                .description(Component.text("「刀不锋利马太瘦，没有实力你别秀」"))
                .description(Component.text("不知怎的，脑子里传来一个身穿旺财，脚踏鬼火的精神小伙的身影"));
        static {
            items.put(0, DISH);
            ids.put(DISH, 0);
            items.put(1, HEART);
            ids.put(HEART, 1);
            items.put(2, STRENGTH);
            ids.put(STRENGTH, 2);
            items.put(3, BADLY_HURT);
            ids.put(BADLY_HURT, 3);
            items.put(4, TOTEM_OF_UNDYING);
            ids.put(TOTEM_OF_UNDYING, 4);
            items.put(5, RESPAWN_BEACON);
            ids.put(RESPAWN_BEACON, 5);
            items.put(6, TREAT);
            ids.put(TREAT, 6);
            items.put(7, MARK);
            ids.put(MARK, 7);
            items.put(8, TURBINE);
            ids.put(TURBINE, 8);
            items.put(9, MILK);
            ids.put(MILK, 9);
            items.put(10, PUNCTURE);
            ids.put(PUNCTURE, 10);
            items.put(11, WHITEWASH);
            ids.put(WHITEWASH, 11);
            items.put(12, SURVEILLANCE);
            ids.put(SURVEILLANCE, 12);
            items.put(13, EVOLUTION_CRYSTAL);
            ids.put(EVOLUTION_CRYSTAL, 13);
            items.put(14, FU_CARD);
            ids.put(FU_CARD, 14);
            items.put(15, TIME_WATCH);
            ids.put(TIME_WATCH, 15);
            items.put(16, PILL);
            ids.put(PILL, 16);
            items.put(17, PUT_OFF);
            ids.put(PUT_OFF, 17);
            items.put(18, CODE);
            ids.put(CODE, 18);
            items.put(19, SEED);
            ids.put(SEED, 19);
            items.put(20, SHIELD);
            ids.put(SHIELD, 20);
            items.put(21, HELMETS_AND_SHOES);
            ids.put(HELMETS_AND_SHOES, 21);
            items.put(22, ARMOR_AND_PANTS);
            ids.put(ARMOR_AND_PANTS, 22);
        }
        public static CustomItem getItem(int id){
            return items.get(id);
        }
        public static Integer getId(CustomItem customItem){
            return ids.get(customItem);
        }
    }
    public final static class ProjectItemStatic {
        public static final CustomItem INTO_INVENTORY_BUTTON = new CustomItem(Material.CHEST, Component.text("道具仓库").color(NamedTextColor.DARK_RED))
                .lore(Component.text("点击以查看获得的道具"));
        public static final CustomItem INVENTORY_BACK_BUTTON = new CustomItem(Material.BLACK_STAINED_GLASS_PANE, Component.text("返回主菜单").color(NamedTextColor.GOLD))
                .lore(Component.text("点击以返回主菜单"));
    }
}
